import * as types from "./actionTypes";
import api from "./api";
import { sendHttpRequest } from "../../redux/actions/httpActions";

export function summonWizard() {
  return { type: types.SUMMON_WIZARD };
}

export function dismissBot() {
  return { type: types.DISMISS_BOT };
}

export function loadBotSession(botName, userKey, botType) {
  return async function(dispatch, getState) {
    try {
      const state = getState();
      const session = state.bot.session[botType];
      if (session && (session.loading || session.loaded)) {
        //a session exists, so check if a chat is open
        if (!state.bot.chat.loaded && !state.bot.chat.loading) {
          dispatch(initializeChat(session.sessionId));
        }
        return;
      }

      dispatch({ type: types.INITIALIZE_BOT_SESSION_STARTED, botType });
      const externalId = state.frontendSession.sessionId;
      const clientApplication = state.frontendSession.applicationCode;
      const data = await dispatch(
        sendHttpRequest(
          api.getBotSession(botName, externalId, clientApplication, userKey)
        )
      );
      dispatch({
        type: types.INITIALIZE_BOT_SESSION_SUCCESS,
        payload: data,
        botType
      });

      dispatch(initializeChat(data.sessionId));
    } catch (error) {
      throw error;
    }
  };
}

function initializeChat(sessionId) {
  return async function(dispatch) {
    try {
      dispatch({ type: types.INITIALIZE_BOT_CHAT_STARTED });
      const data = await dispatch(
        sendHttpRequest(api.initializeChat(sessionId))
      );
      dispatch({
        type: types.INITIALIZE_BOT_CHAT_SUCCESS,
        payload: data
      });
    } catch (error) {
      throw error;
    }
  };
}

export function closeChat() {
  return async function(dispatch, getState) {
    try {
      const { chatId } = getState().bot.chat;
      if (!chatId) return;

      const data = await dispatch(sendHttpRequest(api.closeChat(chatId)));
      dispatch({
        type: types.CLOSE_BOT_CHAT_SUCCESS,
        payload: data
      });
    } catch (error) {
      throw error;
    }
  };
}

export function saveMessage(messageSourceId, messageDate, messageContent) {
  return async function(dispatch, getState) {
    try {
      const { chatId } = getState().bot.chat;
      if (!chatId) {
        //the chat is not yet initialized. The message will be stored on the client and sent to the server next time.
        dispatch({
          type: types.STORE_BOT_MESSAGE,
          message: { messageSourceId, messageDate, messageContent }
        });
        return;
      }

      await dispatch(checkStorage(chatId));
      const event = await dispatch(
        sendHttpRequest(
          api.saveMessage(chatId, messageSourceId, messageDate, messageContent)
        )
      );
      dispatch({ type: types.SAVE_BOT_MESSAGE_SUCCESS, payload: event });
      return event;
    } catch (error) {
      throw error;
    }
  };
}

function checkStorage(chatId) {
  return async function(dispatch, getState) {
    try {
      const messages = getState().bot.storage;
      if (messages.length === 0) return;

      const promises = [];
      messages.forEach(message => {
        const promise = dispatch(
          sendHttpRequest(
            api.saveMessage(
              chatId,
              message.messageSourceId,
              message.messageDate,
              message.messageContent
            )
          )
        );
        promises.push(promise);
      });

      //wait to save all stored messages to keep the order
      await Promise.all(promises);

      //clear stored messages after save
      dispatch({ type: types.CLEAR_BOT_STORAGE });
    } catch (error) {
      throw error;
    }
  };
}
