import React from "react";
import PropTypes from "prop-types";
import { makeStyles } from "@material-ui/core/styles";
import clsx from "clsx";
import {
  Card,
  CardHeader,
  CardContent,
  CardActions,
  Collapse,
  Avatar,
  IconButton,
  Tooltip,
  Menu,
  MenuItem
} from "@material-ui/core";
import ShareIcon from "@material-ui/icons/Share";
import ExpandMoreIcon from "@material-ui/icons/ExpandMore";
import MoreVertIcon from "@material-ui/icons/MoreVert";
import DnsRoundedIcon from "@material-ui/icons/DnsRounded";
import MessageRoundedIcon from "@material-ui/icons/MessageRounded";
import styles from "../../../components/common/styles/expandableCardStyles";
import ServerSummary from "./ServerSummary";
import { useTranslation } from "react-i18next";
import ServerChartsContainer from "../../charts/server/components/ServerChartsContainer";

const useStyles = makeStyles(styles);

const ServerComponent = ({
  data,
  serverHost,
  openAbout,
  handleOpenInNewTab,
  showMessageForAuthor,
  summonWizard
}) => {
  const classes = useStyles();
  const { t } = useTranslation();

  const [expanded, setExpanded] = React.useState(false);
  const [anchorEl, setAnchorEl] = React.useState(null);

  const handleExpandClick = () => {
    setExpanded(!expanded);
  };

  const handleClick = (event) => {
    setAnchorEl(event.currentTarget);
  };

  const handleClose = () => {
    setAnchorEl(null);
  };

  const handleDetailsClick = () => {
    openAbout();
    handleClose();
  };

  return (
    <>
      <Menu
        id="server-details-menu"
        anchorEl={anchorEl}
        keepMounted
        open={Boolean(anchorEl)}
        onClose={handleClose}
      >
        <MenuItem onClick={handleDetailsClick}>{t("Server.Details")}</MenuItem>
      </Menu>
      <Card>
        <CardHeader
          avatar={
            <Avatar aria-label="recipe" className={classes.avatar}>
              <DnsRoundedIcon />
            </Avatar>
          }
          action={
            <IconButton
              aria-label="settings"
              onClick={handleClick}
              color="primary"
            >
              <MoreVertIcon />
            </IconButton>
          }
          title={<strong>{t("Server.Title")}</strong>}
          subheader={t("Server.Subtitle")}
        />
        <CardContent>
          {data.loaded && (
            <ServerSummary
              data={data}
              serverHost={serverHost}
              handleOpenInNewTab={handleOpenInNewTab}
              summonWizard={summonWizard}
            />
          )}
        </CardContent>
        <CardActions disableSpacing>
          <Tooltip title={t("Server.Actions.SendMessageToAuthor")}>
            <IconButton
              aria-label="sent message"
              onClick={showMessageForAuthor}
            >
              <MessageRoundedIcon />
            </IconButton>
          </Tooltip>
          <IconButton aria-label="share">
            <ShareIcon />
          </IconButton>
          <IconButton
            className={clsx(classes.expand, {
              [classes.expandOpen]: expanded
            })}
            onClick={handleExpandClick}
            aria-expanded={expanded}
            aria-label="show more"
          >
            <ExpandMoreIcon />
          </IconButton>
        </CardActions>
        <Collapse in={expanded} timeout="auto" unmountOnExit>
          <CardContent>
            <ServerChartsContainer />
          </CardContent>
        </Collapse>
      </Card>
    </>
  );
};

ServerComponent.propTypes = {
  data: PropTypes.object.isRequired,
  serverHost: PropTypes.string,
  openAbout: PropTypes.func.isRequired,
  handleOpenInNewTab: PropTypes.func.isRequired,
  showMessageForAuthor: PropTypes.func.isRequired,
  summonWizard: PropTypes.func.isRequired
};

export default ServerComponent;
